@@ -1,5 +1,4 @@
-// RUN: %clang_cc1 -no-opaque-pointers -triple i386-pc-linux-gnu -emit-llvm -o %t %s
-// RUN: grep "g.b = internal global i8. getelementptr" %t
+// RUN: %clang_cc1 -triple i386-pc-linux-gnu -emit-llvm -o - %s | FileCheck %s
 
 struct AStruct { 
   int i;
@@ -15,6 +14,7 @@ void f(void) {
   static struct AStruct myStruct = { 1, "two", 3.0 };
 }
 
+// CHECK: @g.b = internal global ptr @g.a
 void g(void) {
   static char a[10];
   static char *b = a;
@@ -26,8 +26,8 @@ void foo(void) {
   static struct s var = {((void*)&((char*)0)[0])};
 }
 
-// RUN: grep "f1.l0 = internal global i32 ptrtoint (i32 ()\* @f1 to i32)" %t
-int f1(void) { static int l0 = (unsigned) f1; }
+// CHECK: @f1.l0 = internal global i32 ptrtoint (ptr @f1 to i32)
+void f1(void) { static int l0 = (unsigned) f1; }
 
 // PR7044
 char *f2(char key) {
